pub struct StopWatch { /* private fields */ }
Measure elapsed time

Implementations

source

impl StopWatch

source

pub fn new() -> StopWatch

Create a new StopWatch

source

pub fn start(&mut self)

Start duration measurement

source

pub fn resume(&mut self)

Resume duration measurement

Examples found in repository?
examples/benchmark_analysis_example.rs (line 26)
22
23
24
25
26
27
28
29
30
31
32
fn example(stop_watch: &mut StopWatch, config: Config, work: u64) -> Result<(), anyhow::Error> {
    stop_watch.pause();
    // perform potentially lengthy preparation that will not reflect in the measurement
    let sleep_time = config.workloads.get(&work).unwrap().clone();
    stop_watch.resume();
    // perform measured computation
    thread::sleep(sleep_time);
    stop_watch.pause();
    // perform potentially lengthy cleanup
    Ok(())
}
source

pub fn reset(&mut self)

Reset duration measurement

source

pub fn stop(&mut self)

Stop duration measurement

source

pub fn pause(&mut self)

Pause duration measurement

Examples found in repository?
examples/benchmark_analysis_example.rs (line 23)
22
23
24
25
26
27
28
29
30
31
32
fn example(stop_watch: &mut StopWatch, config: Config, work: u64) -> Result<(), anyhow::Error> {
    stop_watch.pause();
    // perform potentially lengthy preparation that will not reflect in the measurement
    let sleep_time = config.workloads.get(&work).unwrap().clone();
    stop_watch.resume();
    // perform measured computation
    thread::sleep(sleep_time);
    stop_watch.pause();
    // perform potentially lengthy cleanup
    Ok(())
}
source

pub fn accumulated(&self) -> Duration

Get accumulated duration