Struct benchmark_rs::benchmarks::Benchmarks
source · pub struct Benchmarks<C, W, E>where
C: Clone + Display,
W: Clone + Display,
Error: From<E>,{ /* private fields */ }Expand description
Run and analyze a benchmarks suite
C- configurationW- workload unitE- error type
Implementations
sourceimpl<C, W, E> Benchmarks<C, W, E>where
C: Clone + Display,
W: Clone + Display,
Error: From<E>,
impl<C, W, E> Benchmarks<C, W, E>where C: Clone + Display, W: Clone + Display, Error: From<E>,
sourcepub fn new(name: &str) -> Benchmarks<C, W, E>
pub fn new(name: &str) -> Benchmarks<C, W, E>
Create a new Benchmarks
name- the name of the benchmark session
sourcepub fn add(
&mut self,
name: &str,
f: fn(stop_watch: &mut StopWatch, config: C, workload_point: W) -> Result<(), E>,
config: C,
work: Vec<W>,
repeat: usize,
ramp_up: usize
) -> Result<(), Error>
pub fn add( &mut self, name: &str, f: fn(stop_watch: &mut StopWatch, config: C, workload_point: W) -> Result<(), E>, config: C, work: Vec<W>, repeat: usize, ramp_up: usize ) -> Result<(), Error>
Create and add a Benchmark
name- the name of the benchmark series. The result will be accessible by the name as a key from the summary.f- the function that runs the benchmark.config- the configuration value for this benchmark serieswork- workload points vector for this benchmark series. Elements of this vector are passed tofin each iterationrepeat- number of times the benchmark will be repeatedramp_up- number of times the benchmark will be performed before the measurement is taken
sourcepub fn summary_as_json(&self) -> String
pub fn summary_as_json(&self) -> String
Produce Summary for all series as JSON string.
sourcepub fn csv_headers(&self) -> String
pub fn csv_headers(&self) -> String
Produce headers for Self::summary_as_csv method
sourcepub fn summary_as_csv(&self) -> HashMap<String, Vec<String>>
pub fn summary_as_csv(&self) -> HashMap<String, Vec<String>>
Produce summary for each series as vector of CSV lines. The key for series data is the series name used in Self::add method, values are placed as the headers returned by Self::csv_headers